﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace Optiroute.TSPAlgorithm
{
    /// <summary>
    /// 
    /// </summary>
    public class InputAlgorithmRoute
    {
        public List<Route> InputRoutes { get; set; } = new List<Route>();
        public int DefaultPlaceID { get; set; }
        public int? EndPlaceID { get; set; }
    }
    public class Route
    {
        public int OriginID { get; set; }
        public int DestinationId { get; set; }
        public int Length { get; set; }
    }

    /// <summary>
    /// Vátí matici a list s ID pro pětné zjištění na jakém indexu je která pobočka
    /// </summary>
    public class MatrixRoute
    {
        public int[,] matrixRoute { get; set; }

        public List<int> placeIDs { get; set; } = new List<int>();
        public int DefaultPlaceID { get; set; }
        public int? EndPlaceID { get; set; }

        public MatrixRoute(InputAlgorithmRoute inputAlgoritmRoute)
        {
            int routeQuantity = inputAlgoritmRoute.InputRoutes.Count();
            //kvadratická rovnoce na výpočett toho kolik máme měst když známe počet hran
            int pointsQantity = (1 + (int)Math.Round(Math.Sqrt(1 + 8 * routeQuantity))) / 2;
            matrixRoute = new int[pointsQantity, pointsQantity];

            foreach (Route route in inputAlgoritmRoute.InputRoutes)
            {
                if (!placeIDs.Any(d => d == route.OriginID)) placeIDs.Add(route.OriginID);
            }

            foreach (Route route in inputAlgoritmRoute.InputRoutes)
            {
                if (!placeIDs.Any(d => d == route.DestinationId)) placeIDs.Add(route.DestinationId);
            }

            for (int i = 0; i < pointsQantity; i++)
            {
                for (int j = 0; j < pointsQantity; j++)
                {                   
                    Route route = inputAlgoritmRoute.InputRoutes.SingleOrDefault(d => d.OriginID == placeIDs[i] && d.DestinationId == placeIDs[j]);
                    if (i == j || ((route?.OriginID == inputAlgoritmRoute.EndPlaceID && route?.DestinationId == inputAlgoritmRoute.DefaultPlaceID) || (route?.OriginID == inputAlgoritmRoute.DefaultPlaceID && route?.DestinationId == inputAlgoritmRoute.EndPlaceID))) continue;
                    int? length = route?.Length;
                    if (length == null) continue;
                    matrixRoute[i, j] = length.Value;
                    matrixRoute[j, i] = length.Value;
                }
            }

            this.DefaultPlaceID = inputAlgoritmRoute.DefaultPlaceID;
            this.EndPlaceID = inputAlgoritmRoute.EndPlaceID;
        }
    }

        

    /// <summary>
    /// Třída určující strukturu toho co se bude vracet: Cesta a její délka (v požadovaných jednotkách a travel mode a algoritmu...)
    /// </summary>
    public class RouteTSPResult
    {
        public List<int> AddressisRoute { get; set; } = new List<int>();
        public int Length { get; set; }
        public long TimeCalculation { get; set; }
    }

}
