﻿using System.Text;
using System.Security.Cryptography;
using System.Collections.Generic;
using System.Linq;
using System;

namespace Optiroute.TSPAlgorithm
{
    public abstract class TSPAlgorithmCompute : TSPAlgorithm
    {
        public abstract RouteTSPResult ComputeTSPRoute(MatrixRoute matrixRoute);
        public abstract string Name { get; }
        public abstract string ShortName { get; }
    }

    public abstract class TSPAlgorithmOptimalisation : TSPAlgorithm
    {
        public abstract RouteTSPResult OptimaliseTSPPRoute(MatrixRoute matrixRoute, RouteTSPResult inputRoute);
        public abstract string Name { get; }
        public abstract string ShortName { get; }
    }

    public abstract class TSPAlgorithm
    {
        public class RouteIndex
        {
            public int RouteLentght { get; set; }
            public int FromIndex { get; set; }
            public int ToIndex { get; set; }
        }

        /// <summary>
        /// Převede cestu tvořenou indexy v matici na cestu tvořenou place id
        /// </summary>
        /// <param name="indexRoute">List s cestu reprezentovsnou indexy</param>
        /// <param name="placeIDs">list ktety má na svých indexech place id</param>
        /// <returns>vrací cestu vtvořenou place id</returns>
        protected IEnumerable<int> GetPlaceIDsRouteFromIndex(List<int> indexRoute, List<int> placeIDs)
        {
            foreach (var a in indexRoute) yield return placeIDs[a];
        }
        /// <summary>
        /// Převede cestu tvořenou place id v matici na cestu tvořenou indexy pro hledání v matici 
        /// </summary>
        /// <param name="indexRoute">List s cestu reprezentovsnou place id</param>
        /// <param name="placeIDs">list ktety má na svých indexech place id</param>
        /// <returns>vrací cestu vtvořenou indexy</returns>
        protected IEnumerable<int> GetIndexRouteFromIdsRoute(List<int> idsRoute, List<int> placeIDsBook)
        {
            foreach (var a in idsRoute) yield return placeIDsBook.IndexOf(a);
        }

        protected List<int> GetRightOrderOFPlaceID(List<int> placeIDs, int defaltCityId, int? endCityId)
        {
            if (placeIDs.First() == placeIDs.Last()) placeIDs.RemoveAt(placeIDs.Count() - 1);
            //Hází města se začátku nakonec dokud není na začátku výchozí pobočka 
            Queue<int> MoveQueue(Queue<int> queue)
            {
                while (queue.Peek() != defaltCityId) queue.Enqueue(queue.Dequeue());
                return queue;
            }

            Queue<int> queue = MoveQueue(new Queue<int>(placeIDs));
            var comparequeue = new Queue<int>(queue);
            comparequeue.Dequeue();
            //Pokud je konečná pobočka ve frontě hned za výchotí, tak se fronta přeskupí do opačného směru a posune tak, že výchozí pobočka bude první a konečná poslední
            if (comparequeue.Peek() == endCityId)
            {
                queue = new Queue<int>(queue.Reverse());
                MoveQueue(queue);
            }
            //Pokud nemáme konenou pobočku, tak se chceme vrátit zpět do výchozí
            if (queue.Last() != endCityId && queue.Last() != defaltCityId) queue.Enqueue(defaltCityId);
            return queue.ToList();
        }

        /// <summary>
        /// metoda, která v listu část seřadí opačně
        /// </summary>
        /// <param name="list">List ve kterém ve mají položky převrátit</param>
        /// <param name="indexFromReverse">Index od kterého se mají položky převrátit</param>
        /// <param name="itemNumber">Číslo kolik položek se má převrátit</param>
        protected List<int> ReversePartial(List<int> list, int indexFromReverse, int itemNumber)
        {
            int[] cutPartToReverse = new int[itemNumber];
            for (int i = 0; i < itemNumber; i++)
            {
                cutPartToReverse[i] = list[(indexFromReverse + i) % list.Count()];
            }
            cutPartToReverse = cutPartToReverse.Reverse().ToArray();
            for (int i = 0; i < itemNumber; i++)
            {
                list[(indexFromReverse + i) % list.Count()] = cutPartToReverse[i];
            }
            return list;
        }
        public int GetFullRouteLength(List<int> indexRoute, int[,] lenghtMatrix)
        {
            int lenght = 0;
            for (int i = 0; i < indexRoute.Count; i++)
            {
                lenght = lenght + lenghtMatrix[indexRoute[i], indexRoute[(i + 1) % indexRoute.Count]];
            }
            return lenght;
        }
    }
}
