﻿using System.Collections.Generic;
using System.Linq;
using System.Diagnostics;

namespace Optiroute.TSPAlgorithm
{
    public class TSPAlgorithmGreedy : TSPAlgorithmCompute
    {
        public override string Name => "Hladový algoritmus";
        public override string ShortName => "HA";

        public override RouteTSPResult ComputeTSPRoute(MatrixRoute matrixRoute)
        {
            Stopwatch stopwatch = new Stopwatch();
            stopwatch.Start();
            //Do listu cest přidá všechny cesty z matice cest, cesta si pamatuje svou délku a index z kterého v matici vychází a do která vchází (Bere se v úvahu jen matice kde i<j) 
            List<RouteIndex> indexRoutes = new List<RouteIndex>();
            for (int i = 0; i < matrixRoute.placeIDs.Count(); i++)
            {
                for (int j = 0; j < matrixRoute.placeIDs.Count(); j++)
                {
                    if (i >= j) continue;
                    indexRoutes.Add(new Optiroute.TSPAlgorithm.TSPAlgorithmCompute.RouteIndex()
                    {
                        RouteLentght = matrixRoute.matrixRoute[i, j],
                        FromIndex = i,
                        ToIndex = j,
                    });
                }
            }
            // Seřadí list všech dostupných cest od nejmenší po největší
            List<RouteIndex> orderedIndexRoutes = indexRoutes.OrderBy(d => d.RouteLentght).ToList();

            //pole kde se zaznamenávají skupiny již vybraných dle HA, ošetři se tím aby např. HA nenašel dva malé okruhy namísto jednoho velkého 
            int[] citiesGoups = new int[matrixRoute.placeIDs.Count()];
            //Pole kde se zanamenávají stupně vrcholů grafu 
            int[] citiesDeg = new int[matrixRoute.placeIDs.Count()];
            //List kam se přiřadí cesty vybrané HA
            List<RouteIndex> greedyRoutes = new List<RouteIndex>();
            //List s posloupnosí ID měst pro výstup HA
            List<int> greedyRouteAddressID = new List<int>();

            int counter = 0;
            while (greedyRoutes.Count < matrixRoute.placeIDs.Count())
            {
                int fromRouteIndex = orderedIndexRoutes[counter].FromIndex;
                int toRouteIndex = orderedIndexRoutes[counter].ToIndex;
                int fromGroup = citiesGoups[fromRouteIndex];
                int toGroup = citiesGoups[toRouteIndex];
                if (greedyRoutes.Count == matrixRoute.placeIDs.Count() - 1 && citiesDeg[fromRouteIndex] + 1 <= 2 && citiesDeg[toRouteIndex] + 1 <= 2) { }
                else if (((fromGroup == toGroup && fromGroup != 0) && greedyRoutes.Count != matrixRoute.placeIDs.Count() - 1) || (citiesDeg[fromRouteIndex] + 1 > 2 || citiesDeg[toRouteIndex] + 1 > 2))
                {
                    counter++;
                    continue;
                }

                greedyRoutes.Add(orderedIndexRoutes[counter]);
                citiesDeg[fromRouteIndex]++;
                citiesDeg[toRouteIndex]++;
                if (fromGroup + toGroup == 0)
                {
                    fromGroup = citiesGoups.Max() + 1;
                    toGroup = fromGroup;
                    citiesGoups[fromRouteIndex] = fromGroup;
                    citiesGoups[toRouteIndex] = toGroup;
                }
                else if (fromGroup == 0 && fromGroup != toGroup)
                {
                    fromGroup = toGroup;
                    citiesGoups[fromRouteIndex] = fromGroup;
                    citiesGoups[toRouteIndex] = toGroup;
                }
                else if (toGroup == 0 && fromGroup != toGroup)
                {
                    toGroup = fromGroup;
                    citiesGoups[fromRouteIndex] = fromGroup;
                    citiesGoups[toRouteIndex] = toGroup;
                }
                else
                {
                    for (int i = 0; i < citiesGoups.Count(); i++)
                    {
                        if (citiesGoups[i] == fromGroup) citiesGoups[i] = toGroup;
                    }
                    citiesGoups[fromRouteIndex] = fromGroup;
                    citiesGoups[toRouteIndex] = toGroup;
                }
                counter++;
            }

            Dictionary<int, RouteIndex[]> dictionaryIndexRoutes = new Dictionary<int, RouteIndex[]>();
            
            foreach (var greedyRoute in greedyRoutes)
            {
                AddRouteToPlacesDictionary(dictionaryIndexRoutes, greedyRoute, greedyRoute.FromIndex);
                AddRouteToPlacesDictionary(dictionaryIndexRoutes, greedyRoute, greedyRoute.ToIndex);
            }


            //najdeme první cestu z 0 (inin)
            
            int lastPlaceFrom = dictionaryIndexRoutes[0][0].FromIndex; //prvni cislo je vzdy mensi, takze nula bude na teto pozici
            int lastPlaceTo = dictionaryIndexRoutes[0][0].ToIndex;
            greedyRouteAddressID.Add(lastPlaceFrom);
            greedyRouteAddressID.Add(lastPlaceTo);

            while (true)
            {
                var next = dictionaryIndexRoutes[lastPlaceTo][0].ToIndex == lastPlaceFrom || dictionaryIndexRoutes[lastPlaceTo][0].FromIndex == lastPlaceFrom ? dictionaryIndexRoutes[lastPlaceTo][1] : dictionaryIndexRoutes[lastPlaceTo][0];
                lastPlaceFrom = lastPlaceTo;
                lastPlaceTo = next.FromIndex == lastPlaceFrom ? next.ToIndex : next.FromIndex;
                greedyRouteAddressID.Add(lastPlaceTo);
                if (lastPlaceTo == 0) break;
            }

            greedyRouteAddressID = greedyRouteAddressID.Select(d => matrixRoute.placeIDs[d]).ToList();
            greedyRouteAddressID.Reverse();
            if (greedyRouteAddressID.Any(d => d == matrixRoute.EndPlaceID)) greedyRouteAddressID.RemoveAt(greedyRouteAddressID.Count - 1);

            stopwatch.Stop();
            return new RouteTSPResult()
            {
                AddressisRoute = greedyRouteAddressID,
                Length = greedyRoutes.Sum(d => d.RouteLentght),
                TimeCalculation = stopwatch.ElapsedMilliseconds
            };
        }

        private void AddRouteToPlacesDictionary(Dictionary<int, RouteIndex[]> dictionaryIndexRoute, RouteIndex route, int place)
        {
            if (!dictionaryIndexRoute.TryGetValue(place, out RouteIndex[] value))
                dictionaryIndexRoute.Add(place, new RouteIndex[2] { route, null });
            else value[1] = route;
        }
    }
}
