﻿using System.Collections.Generic;
using System.Linq;
using System.Diagnostics;

namespace Optiroute.TSPAlgorithm
{
    public class TSPAlgorithmNearestNeighbour : TSPAlgorithmCompute
    {
        public override string Name => "Metoda nejbližšího souseda";
        public override string ShortName => "NS";

        public override RouteTSPResult ComputeTSPRoute(MatrixRoute matrixRoute)
        {
            Stopwatch stopwatch = new Stopwatch();
            stopwatch.Start();

            List<NSRoute> variousNSRoutes = new List<NSRoute>();

            for (int i = 0; i < matrixRoute.placeIDs.Count(); i++)
            {
                int routeLength = 0;
                Stack<int> indexRoute = new Stack<int>(matrixRoute.placeIDs.Count());

                indexRoute.Push(i);
                FindNextCity(indexRoute, routeLength, variousNSRoutes, matrixRoute);
            }

            var finalLenght = variousNSRoutes.Min(d => d.NSRouteLenght);
            var finalNSRoute = variousNSRoutes.FirstOrDefault(d => d.NSRouteLenght == finalLenght).IndexNSRoute;

            stopwatch.Stop();
            return new RouteTSPResult()
            {
                TimeCalculation = stopwatch.ElapsedMilliseconds,
                AddressisRoute = GetRightOrderOFPlaceID(GetPlaceIDsRouteFromIndex(finalNSRoute, matrixRoute.placeIDs).ToList(), matrixRoute.DefaultPlaceID, matrixRoute.EndPlaceID),
                Length = finalLenght
            };
        }

        private void FindNextCity(Stack<int> indexRoute, int routeLength, List<NSRoute> variousNSRoutes, MatrixRoute matrixRoute)
        {
            List<int> minIndexes = new List<int>();
            int minLength = int.MaxValue;

            for (int j = 0; j < matrixRoute.placeIDs.Count(); j++)
            {
                if (indexRoute.Any(d => d == j)) continue;
                int length = matrixRoute.matrixRoute[indexRoute.Peek(),j];

                // podmínka zajišťuje aby počáteční a konečná pobočka byly u sebe i v případě, že v matici je další cesta délky 0 vycházející z výchzí nebo konečný pobočly
                // Pokud je cesta délky 0 && navazujeme na výchozí nebo konečnou pobočku && NEmáme v cestě už zařazenou jak výchozí tak konečnou pobočku zároveň&& snažíme se dostat do JINÉ než konečné nebo výchozí pobočky
                if (length == 0 && (indexRoute.Peek() == matrixRoute.DefaultPlaceID || indexRoute.Peek() == matrixRoute.EndPlaceID) && indexRoute.Where(d => d == matrixRoute.DefaultPlaceID || d == matrixRoute.EndPlaceID).Count() < 2 && !(j == matrixRoute.DefaultPlaceID || j == matrixRoute.EndPlaceID)) 
                    continue;
                if (length < minLength)
                {
                    minLength = length;
                    minIndexes = new List<int>() { j }; 
                }
                else if (length == minLength) 
                {
                    minIndexes.Add(j);
                }
            }

            foreach (var a in minIndexes)
            {
                indexRoute.Push(a);

                if (indexRoute.Count() == matrixRoute.placeIDs.Count())
                {
                    variousNSRoutes.Add(new NSRoute() { IndexNSRoute = indexRoute.ToList(), NSRouteLenght = routeLength + minLength + matrixRoute.matrixRoute[indexRoute.Peek(), indexRoute.Last()] });                  
                }
                else FindNextCity(indexRoute, routeLength + minLength, variousNSRoutes, matrixRoute);

                indexRoute.Pop();
            }
        }
    }
    public class NSRoute
    {
        public List<int> IndexNSRoute { get; set; } = new List<int>();
        public int NSRouteLenght { get; set; }
    }
}
