﻿using System.Collections.Generic;
using System.Linq;
using System;
using System.Diagnostics;

namespace Optiroute.TSPAlgorithm
{
    public class Tsp2_OptAlgorithm : TSPAlgorithmOptimalisation
    {
        public override string Name => "2-opt";
        public override string ShortName => "2-opt";
        public override RouteTSPResult OptimaliseTSPPRoute(MatrixRoute matrixRoute, RouteTSPResult inputRoute)
        {
            Stopwatch stopwatch = new Stopwatch();
            stopwatch.Start();

            if (inputRoute.AddressisRoute.Last() == matrixRoute.DefaultPlaceID) inputRoute.AddressisRoute.RemoveAt(inputRoute.AddressisRoute.Count - 1);
            List<int> indexRoute = GetIndexRouteFromIdsRoute(inputRoute.AddressisRoute, matrixRoute.placeIDs).ToList();
            List<TwoRoutes> cobinationsOFTwoRoutes = new List<TwoRoutes>();

            int maxSkipRoute = Convert.ToInt32(Math.Floor((matrixRoute.placeIDs.Count() - 2) / 2d));
            if (matrixRoute.placeIDs.Count() == 3) maxSkipRoute = 0;

            for (int i = 1; i <= maxSkipRoute; i++)
            {
                int combinationNumber = matrixRoute.placeIDs.Count();
                if (combinationNumber % 2 == 0 && i == maxSkipRoute) combinationNumber = combinationNumber / 2;
                for (int j = 0; j < combinationNumber; j++)
                {
                    cobinationsOFTwoRoutes.Add(new TwoRoutes()
                    {
                        FromRoute1 = j,
                        ToRoute1 = (j + 1) % matrixRoute.placeIDs.Count(),
                        FromRoute2 = (j + 1 + i) % matrixRoute.placeIDs.Count(),
                        ToRoute2 = (j + 1 + i + 1) % matrixRoute.placeIDs.Count()
                    });
                }
            }

            bool processing = true;
            while(processing)
            {
                processing = false;
                foreach (var a in cobinationsOFTwoRoutes)
                {
                    int oldTwoRoutesLength = matrixRoute.matrixRoute[indexRoute[a.FromRoute1], indexRoute[a.ToRoute1]] + matrixRoute.matrixRoute[indexRoute[a.FromRoute2], indexRoute[a.ToRoute2]];
                    int newTwoRotesLength = matrixRoute.matrixRoute[indexRoute[a.FromRoute1], indexRoute[a.FromRoute2]] + matrixRoute.matrixRoute[indexRoute[a.ToRoute1], indexRoute[a.ToRoute2]];
                    if (newTwoRotesLength < oldTwoRoutesLength)
                    {
                        int xchangeFromRoute2 = indexRoute[a.FromRoute2];
                        int xchangeToRoute1 = indexRoute[a.ToRoute1];
                        indexRoute[a.ToRoute1] = xchangeFromRoute2;
                        indexRoute[a.FromRoute2] = xchangeToRoute1;
                        int skippedCities = ((a.FromRoute2 + indexRoute.Count() - a.ToRoute1) % indexRoute.Count()) - 1;
                        if (skippedCities >= 2)indexRoute = ReversePartial(indexRoute, a.ToRoute1 + 1, skippedCities);
                        processing = true;
                    }
                }
            }

            stopwatch.Stop();
            return new RouteTSPResult()
            {
                TimeCalculation = stopwatch.ElapsedMilliseconds,
                AddressisRoute = GetRightOrderOFPlaceID(GetPlaceIDsRouteFromIndex(indexRoute, matrixRoute.placeIDs).ToList(), matrixRoute.DefaultPlaceID, matrixRoute.EndPlaceID),
                Length = GetFullRouteLength(indexRoute, matrixRoute.matrixRoute)
            };
        }

        private class TwoRoutes
        {
            public int FromRoute1 { get; set; }
            public int ToRoute1 { get; set; }
            public int FromRoute2 { get; set; }
            public int ToRoute2 { get; set; }
        }
    }
}
