﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;
using System.Threading.Tasks;
using System.Linq;
using Newtonsoft.Json;
using System.IO;
using Optiroute.Db;
using Optiroute.TSPAlgorithm;

namespace Optiroute.DataLoader.Google.Tests
{
    [TestClass]
    public class FindAddressTests
    {
        string _googleApiKey = "AIzaSyBdUmJ-lpaNjGysMFSR_52NYhYWA0ititw";
        string _googleBaseURL = "https://maps.googleapis.com/maps/api";
        [TestMethod]
        public async Task FindAddress1()
        {
            AddressGoogleServices addressService = new AddressGoogleServices(_googleApiKey, _googleBaseURL);
            var result = await addressService.RecognizeAddress("Sokolovská 1017/252");
            Assert.AreEqual("Sokolovská 1017, 190 00 Praha 9-Libeň, Czechia", result.FullAdress);
            Assert.AreEqual("ChIJByfn_bLsC0cRTenMBaGYp6A", result.PlaceID);
        }
        [TestMethod]
        public async Task FindRoute()
        {
            AddressGoogleServices addressService = new AddressGoogleServices(_googleApiKey, _googleBaseURL);
            List<string> origins = new List<string>() {"place_id:ChIJGV15QbTsC0cRcdeEGEHl4N8"};
            List<string> destinations = new List<string>() {"place_id:ChIJs4h8LAQ1CUcRC1N2hm1zxWE"};
            FindRouteModel addresisToVisit = new FindRouteModel(origins, destinations, TravelMode.Transit);
            var result = await addressService.FindRouteLenght(addresisToVisit);
            Assert.AreEqual("K Moravině 1871, 190 00 Praha 9-Libeň, Czechia", result.Routes[0].Origin.RecognizedAddress);
            Assert.AreEqual(origins.First(), result.Routes.First().Origin.OriginalAddress);
            Assert.AreEqual("Prosečská 231, Proseč nad Nisou, 468 04 Jablonec nad Nisou, Czechia", result.Routes[0].Destination.RecognizedAddress);
            Assert.AreEqual(destinations.First(), result.Routes.First().Destination.OriginalAddress);
            Assert.AreEqual(103681, result.Routes.First().DistanceMeters);
            Assert.AreEqual(4041, result.Routes.First().DurationTime);

            Directory.CreateDirectory($@"C:\data");
            await File.WriteAllTextAsync($@"C:\data\results.json", JsonConvert.SerializeObject(result.Routes));
        }
        [TestMethod]
        public async Task CreateDbTest()
        {
            DbContext jsonDb = new DbContext();
            jsonDb.Places.Add(new Optiroute.Db.Model.Place()
            {
                Id = 1,
                NickName = "ININ",
                Address = "K Moravině 1871, 190 00 Praha 9-Libeň",
                AddressGoogleId = "ChIJGV15QbTsC0cRcdeEGEHl4N8",
                Default = true,
                FullAddress = "K Moravině 1871, 190 00 Praha 9-Libeň, Czechia"
            });

            jsonDb.Places.Add(new Optiroute.Db.Model.Place()
            {
                Id = 2,
                NickName = "Proseč",
                Address = "Prosečská 231, Proseč nad Nisou, 468 04 Jablonec nad Nisou",
                AddressGoogleId = "ChIJs4h8LAQ1CUcRC1N2hm1zxWE",
                Default = false,
                FullAddress = "Prosečská 231, Proseč nad Nisou, 468 04 Jablonec nad Nisou, Czechia"
            });

            jsonDb.Routes.Add(new Optiroute.Db.Model.Route()
            {
                Id = 1,
                OriginId = 1,
                DestinationId = 2,
                MeasuresLenght = new List<Optiroute.Db.Model.Measure>()
                {
                    new Db.Model.Measure() { Duration = 4041, Distance = 103681, TravelMode = Optiroute.Db.Model.TravelMode.Transit },
                    new Db.Model.Measure() { Duration = 5041, Distance = 103000, TravelMode = Optiroute.Db.Model.TravelMode.Bicycling },
                    new Db.Model.Measure() { Duration = 3041, Distance = 103600, TravelMode = Optiroute.Db.Model.TravelMode.Driving },
                    new Db.Model.Measure() { Duration = 10041, Distance = 93681, TravelMode = Optiroute.Db.Model.TravelMode.Walking }
                }
            });

            await jsonDb.SaveToFile($@"C:\data\Db.json");
        }

        [TestMethod]
        public async Task CreateFullmatrixEU()
        {
            throw new ApplicationException("To stojí peníze! Opravdu mě chceš spustit? To mě musíš zakomentovat!");
            DbContext jsonDb = new DbContext()
            {
                Places = new List<Db.Model.Place>()
                {
                    new Db.Model.Place(){ Id = 0, NickName = "Praha, Česko", Address = "Praha, Česko", FullName = "Praha, Česko", Default = true },
                    new Db.Model.Place(){ Id = 1, NickName = "Tirana, Albánie", Address = "Tirana, Albánie", FullName = "Tirana, Albánie",},
                    new Db.Model.Place(){ Id = 2, NickName = "Andorra la Vella, Andorra", Address = "Andorra la Vella, Andorra", FullName = "Andorra la Vella, Andorra",},
                    new Db.Model.Place(){ Id = 3, NickName = "Jerevan, Arménie", Address = "Jerevan, Arménie", FullName = "Jerevan, Arménie",},
                    new Db.Model.Place(){ Id = 4, NickName = "Baku, Ázerbájdžán", Address = "Baku, Ázerbájdžán", FullName = "Baku, Ázerbájdžán",},
                    new Db.Model.Place(){ Id = 5, NickName = "Bruselas, Belgie", Address = "Bruselas, Belgie", FullName = "Bruselas, Belgie",},
                    new Db.Model.Place(){ Id = 6, NickName = "Minsk, Bělorusko", Address = "Minsk, Bělorusko", FullName = "Minsk, Bělorusko",},
                    new Db.Model.Place(){ Id = 7, NickName = "Sarajevo, Bosna a Hercegovina", Address = "Sarajevo, Bosna a Hercegovina", FullName = "Sarajevo, Bosna a Hercegovina",},
                    new Db.Model.Place(){ Id = 8, NickName = "Sofie, Bulharsko", Address = "Sofie, Bulharsko", FullName = "Sofie, Bulharsko",},
                    new Db.Model.Place(){ Id = 9, NickName = "Podgorica, Černá Hora", Address = "Podgorica, Černá Hora", FullName = "Podgorica, Černá Hora",},
                    new Db.Model.Place(){ Id = 10, NickName = "Kodaň, Dánsko", Address = "Kodaň, Dánsko", FullName = "Kodaň, Dánsko",},
                    new Db.Model.Place(){ Id = 11, NickName = "Tallinn, Estonsko", Address = "Tallinn, Estonsko", FullName = "Tallinn, Estonsko",},
                    new Db.Model.Place(){ Id = 12, NickName = "Helsinki, Finsko", Address = "Helsinki, Finsko", FullName = "Helsinki, Finsko",},
                    new Db.Model.Place(){ Id = 13, NickName = "Paříž, Francie", Address = "Paříž, Francie", FullName = "Paříž, Francie",},
                    new Db.Model.Place(){ Id = 14, NickName = "Tbilisi, Gruzie", Address = "Tbilisi, Gruzie", FullName = "Tbilisi, Gruzie",},
                    new Db.Model.Place(){ Id = 15, NickName = "Záhřeb, Chorvatsko", Address = "Záhřeb, Chorvatsko", FullName = "Záhřeb, Chorvatsko",},
                    new Db.Model.Place(){ Id = 16, NickName = "Dublin, Irsko", Address = "Dublin, Irsko", FullName = "Dublin, Irsko",},
                    new Db.Model.Place(){ Id = 17, NickName = "Reykjavík, Island", Address = "Reykjavík, Island", FullName = "Reykjavík, Island",},
                    new Db.Model.Place(){ Id = 18, NickName = "Řím, Itálie", Address = "Řím, Itálie", FullName = "Řím, Itálie",},
                    new Db.Model.Place(){ Id = 19, NickName = "Nur-Sultan, Kazachstán", Address = "Nur-Sultan, Kazachstán", FullName = "Nur-Sultan, Kazachstán",},
                    new Db.Model.Place(){ Id = 20, NickName = "Nikósie, Kypr", Address = "Nikósie, Kypr", FullName = "Nikósie, Kypr",},
                    new Db.Model.Place(){ Id = 21, NickName = "Vaduz, Lichtenštejnsko", Address = "Vaduz, Lichtenštejnsko", FullName = "Vaduz, Lichtenštejnsko"},
                    new Db.Model.Place(){ Id = 22, NickName = "Vilnius, Litva", Address = "Vilnius, Litva", FullName = "Vilnius, Litva"},
                    new Db.Model.Place(){ Id = 23, NickName = "Riga, Lotyšsko", Address = "Riga, Lotyšsko", FullName = "Riga, Lotyšsko"},
                    new Db.Model.Place(){ Id = 24, NickName = "Lucemburk, Lucembursko", Address = "Lucemburk, Lucembursko", FullName = "Lucemburk, Lucembursko"},
                    new Db.Model.Place(){ Id = 25, NickName = "Budapešť, Maďarsko", Address = "Budapešť, Maďarsko", FullName = "Budapešť, Maďarsko"},
                    new Db.Model.Place(){ Id = 26, NickName = "Valletta, Malta", Address = "Valletta, Malta", FullName = "Valletta, Malta"},
                    new Db.Model.Place(){ Id = 27, NickName = "Kišiněv, Moldavsko", Address = "Kišiněv, Moldavsko", FullName = "Kišiněv, Moldavsko"},
                    new Db.Model.Place(){ Id = 28, NickName = "Monaco-Ville, Monako", Address = "Monaco-Ville, Monako", FullName = "Monaco-Ville, Monako"},
                    new Db.Model.Place(){ Id = 29, NickName = "Berlín, Německo", Address = "Berlín, Německo", FullName = "Berlín, Německo"},
                    new Db.Model.Place(){ Id = 30, NickName = "Amsterdam, Nizozemsko", Address = "Amsterdam, Nizozemsko", FullName = "Amsterdam, Nizozemsko"},
                    new Db.Model.Place(){ Id = 31, NickName = "Oslo, Norsko", Address = "Oslo, Norsko", FullName = "Oslo, Norsko"},
                    new Db.Model.Place(){ Id = 32, NickName = "Varšava, Polsko", Address = "Varšava, Polsko", FullName = "Varšava, Polsko"},
                    new Db.Model.Place(){ Id = 33, NickName = "Lisabon, Portugalsko", Address = "Lisabon, Portugalsko", FullName = "Lisabon, Portugalsko"},
                    new Db.Model.Place(){ Id = 34, NickName = "Vídeň, Rakousko", Address = "Vídeň, Rakousko", FullName = "Vídeň, Rakousko"},
                    new Db.Model.Place(){ Id = 35, NickName = "Bukurešť, Rumunsko", Address = "Bukurešť, Rumunsko", FullName = "Bukurešť, Rumunsko"},
                    new Db.Model.Place(){ Id = 36, NickName = "Moskva, Rusko", Address = "Moskva, Rusko", FullName = "Moskva, Rusko"},
                    new Db.Model.Place(){ Id = 37, NickName = "Atény, Řecko", Address = "Atény, Řecko", FullName = "Atény, Řecko"},
                    new Db.Model.Place(){ Id = 38, NickName = "San Marino, San Marino", Address = "San Marino, San Marino", FullName = "San Marino, San Marino"},
                    new Db.Model.Place(){ Id = 39, NickName = "Skopje, Severní Makedonie", Address = "Skopje, Severní Makedonie", FullName = "Skopje, Severní Makedonie"},
                    new Db.Model.Place(){ Id = 40, NickName = "Bratislava, Slovensko", Address = "Bratislava, Slovensko", FullName = "Bratislava, Slovensko"},
                    new Db.Model.Place(){ Id = 41, NickName = "Lublaň, Slovinsko", Address = "Lublaň, Slovinsko", FullName = "Lublaň, Slovinsko"},
                    new Db.Model.Place(){ Id = 42, NickName = "Londýn, Velká Británie", Address = "Londýn, Velká Británie", FullName = "Londýn, Velká Británie"},
                    new Db.Model.Place(){ Id = 43, NickName = "Bělehrad, Srbsko", Address = "Bělehrad, Srbsko", FullName = "Bělehrad, Srbsko"},
                    new Db.Model.Place(){ Id = 44, NickName = "Madrid, Španělsko", Address = "Madrid, Španělsko", FullName = "Madrid, Španělsko"},
                    new Db.Model.Place(){ Id = 45, NickName = "Stockholm, Švédsko", Address = "Stockholm, Švédsko", FullName = "Stockholm, Švédsko"},
                    new Db.Model.Place(){ Id = 46, NickName = "Bern, Švýcarsko", Address = "Bern, Švýcarsko", FullName = "Bern, Švýcarsko"},
                    new Db.Model.Place(){ Id = 47, NickName = "Ankara, Turecko", Address = "Ankara, Turecko", FullName = "Ankara, Turecko"},
                    new Db.Model.Place(){ Id = 48, NickName = "Kyjev, Ukrajina", Address = "Kyjev, Ukrajina", FullName = "Kyjev, Ukrajina"},
                    new Db.Model.Place(){ Id = 49, NickName = "Vatikán", Address = "Vatikán", FullName = "Vatikán"},
                }
            };

            await GenerateJSON(jsonDb);
        }

        /// <summary>
        /// Test pro vyvoření všech cest mezi všema pobočkama
        /// </summary>
        /// <returns>Json všech cest mezi všema pobočkama</returns>
        [TestMethod]
        public async Task CreateFullmatrixININ()
        {
            throw new ApplicationException("To stojí peníze! Opravdu mě chceš spustit? To mě musíš zakomentovat!");
            DbContext jsonDb = new DbContext()
            {
                Places = new List<Db.Model.Place>()
                {
                    new Db.Model.Place(){ Id = 0, NickName = "In Investments", Address = "K Moravině 1871, 190 00 Praha 9-Libeň", FullName = "In Investments s. r. o.", Default = true },
                    new Db.Model.Place(){ Id = 1, NickName = "Amundi", Address = "Rohanské nábř. 693, 186 00 Praha 8-Karlín", FullName = "Amundi Czech Republic, investiční společnost, a.s.",},
                    new Db.Model.Place(){ Id = 2, NickName = "Avant", Address = "City Tower, Hvězdova, 140 00 Praha 4-Nusle", FullName = "AVANT investiční společnost, a.s.",},
                    new Db.Model.Place(){ Id = 3, NickName = "Conseq", Address = "Rybná 682, 110 00 Staré Město", FullName = "Conseq Investment Management, a.s. & Conseq penzijní společnost, a.s.",},
                    new Db.Model.Place(){ Id = 4, NickName = "Efekta", Address = "náměstí Svobody 91/20, 602 00 Brno", FullName = "EFEKTA obchodník s cennými papíry a.s.",},
                    new Db.Model.Place(){ Id = 5, NickName = "EIC", Address = "Veveří 2581/102, 616 00 Brno - Žabovřesky, Česko", FullName = "European Investment Centre o.c.p., a.s., (EIC)",},
                    new Db.Model.Place(){ Id = 6, NickName = "Generali", Address = "Na Pankráci 1720/123, 140 21 Praha 4", FullName = "Generali penzijní / investiční společnost, a.s.",},
                    new Db.Model.Place(){ Id = 7, NickName = "Hello bank!", Address = "Karla Engliše 3208/5, 150 00 Praha 5-Smíchov", FullName = "Hello bank!",},
                    new Db.Model.Place(){ Id = 8, NickName = "INVESTIKA", Address = "U Zvonařky 291, 120 00 Vinohrady", FullName = "INVESTIKA, investiční společnost, a.s.",},
                    new Db.Model.Place(){ Id = 9, NickName = "J&T Banka", Address = "Pobřežní 14, 186 00 Praha 8", FullName = "J&T Banka, a.s.",},
                    new Db.Model.Place(){ Id = 10, NickName = "Moventum", Address = "Palác Padowetz, Bašty 413/2, 602 00 Brno", FullName = "Moventum a.s.",},
                    new Db.Model.Place(){ Id = 11, NickName = "Tesla", Address = "Konviktská 291/24, 110 00 Staré Město", FullName = "Tesla investiční společnost, a.s.",},
                    new Db.Model.Place(){ Id = 12, NickName = "Artesa", Address = "28. října 212/37, Ostrava, 702 00", FullName = "Artesa, spořitelní družstvo",},
                    new Db.Model.Place(){ Id = 13, NickName = "EKKA-Gold", Address = "Kostelní 89/13, 702 00 Moravská Ostrava a Přívoz", FullName = "EKKA-Gold, s.r.o.",},
                    new Db.Model.Place(){ Id = 14, NickName = "Allianz", Address = "Ke Štvanici 656/3, 186 00 Karlín", FullName = "Allianz penzijní společnost, a.s.",},
                    new Db.Model.Place(){ Id = 15, NickName = "ČSOB", Address = "Radlická 333/150, 150 00 Praha 5", FullName = "ČSOB Penzijní společnost, a. s.",},
                    new Db.Model.Place(){ Id = 16, NickName = "Česká spořitelna", Address = "Poláčkova 1976, 140 00 Praha 4-Krč", FullName = "Česká spořitelna — penzijní společnost, a.s.",},
                    new Db.Model.Place(){ Id = 17, NickName = "KB", Address = "Bílinská 175/2 , Ústí nad Labem, 400 42", FullName = "KB Penzijní společnost, a.s.",},
                    new Db.Model.Place(){ Id = 18, NickName = "NN", Address = "Nádražní 344, 150 00 Praha 5-Smíchov", FullName = "NN Penzijní společnost, a.s.",},
                    new Db.Model.Place(){ Id = 19, NickName = "Uniqa", Address = "Úzká 488, 602 00 Brno-střed-Trnitá", FullName = "Uniqa penzijní společnost a.s.",},
                    new Db.Model.Place(){ Id = 20, NickName = "Insia", Address = "Vinohradská 2828, 130 00 Praha 3-Žižkov", FullName = "INSIA a.s.",},
                    new Db.Model.Place(){ Id = 21, NickName = "Domov", Address = "Říčany Vrchlického", FullName = "Bydliště - Konečná adresa", EndCity = true},
                }
            };

            await GenerateJSON(jsonDb);
        }

        [TestMethod]
        public async Task CreateFullmatrixININBefore()
        {
            //throw new ApplicationException("To stojí peníze! Opravdu mě chceš spustit? To mě musíš zakomentovat!");
            DbContext jsonDb = new DbContext()
            {
                Places = new List<Db.Model.Place>()
                {
                    new Db.Model.Place(){ Id = 0, NickName = "In Investments", Address = "K Moravině 1871, 190 00 Praha 9-Libeň", FullName = "In Investments s. r. o.", Default = true },
                    new Db.Model.Place(){ Id = 1, NickName = "Amundi", Address = "Rohanské nábř. 693, 186 00 Praha 8-Karlín", FullName = "Amundi Czech Republic, investiční společnost, a.s.",},
                    new Db.Model.Place(){ Id = 2, NickName = "Avant", Address = "Rohanské nábřeží 671/15, 186 00 Praha, Karlín", FullName = "AVANT investiční společnost, a.s.",},
                    new Db.Model.Place(){ Id = 3, NickName = "Conseq", Address = "Rybná 682, 110 00 Staré Město", FullName = "Conseq Investment Management, a.s. & Conseq penzijní společnost, a.s.",},
                    new Db.Model.Place(){ Id = 4, NickName = "Efekta", Address = "náměstí Svobody 91/20, 602 00 Brno", FullName = "EFEKTA obchodník s cennými papíry a.s.",},
                    new Db.Model.Place(){ Id = 5, NickName = "EIC", Address = "Veveří 2581/102, 616 00 Brno - Žabovřesky, Česko", FullName = "European Investment Centre o.c.p., a.s., (EIC)",},
                    new Db.Model.Place(){ Id = 6, NickName = "Generali", Address = "Na Pankráci 1720/123, 140 21 Praha 4", FullName = "Generali penzijní / investiční společnost, a.s.",},
                    new Db.Model.Place(){ Id = 7, NickName = "Hello bank!", Address = "Karla Engliše 3208/5, 150 00 Praha 5-Smíchov", FullName = "Hello bank!",},
                    new Db.Model.Place(){ Id = 8, NickName = "INVESTIKA", Address = "U Zvonařky 291, 120 00 Vinohrady", FullName = "INVESTIKA, investiční společnost, a.s.",},
                    new Db.Model.Place(){ Id = 9, NickName = "J&T Banka", Address = "Pobřežní 14, 186 00 Praha 8", FullName = "J&T Banka, a.s.",},
                    new Db.Model.Place(){ Id = 10, NickName = "Moventum", Address = "Palác Padowetz, Bašty 413/2, 602 00 Brno", FullName = "Moventum a.s.",},
                    new Db.Model.Place(){ Id = 11, NickName = "Tesla", Address = "Konviktská 291/24, 110 00 Staré Město", FullName = "Tesla investiční společnost, a.s.",},
                    new Db.Model.Place(){ Id = 12, NickName = "Artesa", Address = "28. října 212/37, Ostrava, 702 00", FullName = "Artesa, spořitelní družstvo",},
                    new Db.Model.Place(){ Id = 13, NickName = "EKKA-Gold", Address = "Kostelní 89/13, 702 00 Moravská Ostrava a Přívoz", FullName = "EKKA-Gold, s.r.o.",},
                    new Db.Model.Place(){ Id = 14, NickName = "Allianz", Address = "Ke Štvanici 656/3, 186 00 Karlín", FullName = "Allianz penzijní společnost, a.s.",},
                    new Db.Model.Place(){ Id = 15, NickName = "ČSOB", Address = "Radlická 333/150, 150 00 Praha 5", FullName = "ČSOB Penzijní společnost, a. s.",},
                    new Db.Model.Place(){ Id = 16, NickName = "Česká spořitelna", Address = "Poláčkova 1976, 140 00 Praha 4-Krč", FullName = "Česká spořitelna — penzijní společnost, a.s.",},
                    new Db.Model.Place(){ Id = 17, NickName = "KB", Address = "Bílinská 175/2 , Ústí nad Labem, 400 42", FullName = "KB Penzijní společnost, a.s.",},
                    new Db.Model.Place(){ Id = 18, NickName = "NN", Address = "Nádražní 344, 150 00 Praha 5-Smíchov", FullName = "NN Penzijní společnost, a.s.",},
                    new Db.Model.Place(){ Id = 19, NickName = "Uniqa", Address = "Úzká 488, 602 00 Brno-střed-Trnitá", FullName = "Uniqa penzijní společnost a.s.",},
                    new Db.Model.Place(){ Id = 20, NickName = "Insia", Address = "Vinohradská 2828, 130 00 Praha 3-Žižkov", FullName = "INSIA a.s.",},
                    new Db.Model.Place(){ Id = 21, NickName = "Domov", Address = "Vltavská 22, Praha 5-Anděl", FullName = "Bydliště - Konečná adresa", EndCity = true},
                }
            };

            await GenerateJSON(jsonDb);
        }

        private Db.Model.Measure GetMesasure(RouteMatrixResult resultDriving, string originAddress, string destinationAddress, Db.Model.TravelMode mode)
        {
            var routeResult = resultDriving.Routes.Single(d => d.Origin.OriginalAddress == originAddress && d.Destination.OriginalAddress == destinationAddress);
            return new Db.Model.Measure()
            {
                TravelMode = mode,
                Distance = routeResult.DistanceMeters,
                Duration = routeResult.DurationTime
            };
        }

        private List<string> MapPlace(List<Db.Model.Place> places)
        {
            return places.Select(place => place.AddressGoogleId).ToList();
        }

        public async Task<MatrixRoute> GetMatrixRoute(String file, Db.Model.TravelMode travelMode, Db.Model.WeightMode weight)
        {
            DbContext fileRoutes = await DbContext.LoadFromFile(file);
            InputAlgorithmRoute algoRoute = new InputAlgorithmRoute();
            algoRoute.InputRoutes = new List<TSPAlgorithm.Route>();

            foreach (Db.Model.Route route in fileRoutes.Routes)
            {
                var measure = route.MeasuresLenght.SingleOrDefault(d => d.TravelMode == travelMode);
                algoRoute.InputRoutes.Add(new TSPAlgorithm.Route()
                {
                    DestinationId = route.DestinationId,
                    OriginID = route.OriginId,
                    Length = weight == Db.Model.WeightMode.Duration ? measure.Duration : measure.Distance,
                     
                }); 
            }

            algoRoute.DefaultPlaceID = fileRoutes.Places.SingleOrDefault(d => d.Default).Id;
            algoRoute.EndPlaceID = fileRoutes.Places.SingleOrDefault(d => d.EndCity).Id;

            return new MatrixRoute(algoRoute);
        }
        private async Task GenerateJSON(DbContext jsonDb)
        {
            AddressGoogleServices addressService = new AddressGoogleServices(_googleApiKey, _googleBaseURL);

            foreach (var a in jsonDb.Places)
            {
                var result = await addressService.RecognizeAddress(a.Address);
                a.FullAddress = result.FullAdress;
                a.AddressGoogleId = $"place_id:{result.PlaceID}";
            }

            int currentRouteId = 1;
            //Přiřadí všechny možnosti cest všech poboček do db tak aby se žádná cesta nepočítala vícekrát
            for (int i = 0; i < jsonDb.Places.Count() - 1; i++)
            {
                List<Db.Model.Place> origins = new List<Db.Model.Place>() { jsonDb.Places[i] };
                List<Db.Model.Place> destinations = new List<Db.Model.Place>();
                for (int j = i + 1; j < jsonDb.Places.Count(); j++)
                {
                    destinations.Add(jsonDb.Places[j]);
                }
                var resultDriving = await addressService.FindRouteLenght(new FindRouteModel(MapPlace(origins), MapPlace(destinations), TravelMode.Driving));
                var resultTransit = await addressService.FindRouteLenght(new FindRouteModel(MapPlace(origins), MapPlace(destinations), TravelMode.Transit));
                var resultBicyclink = await addressService.FindRouteLenght(new FindRouteModel(MapPlace(origins), MapPlace(destinations), TravelMode.Bicycling));
                var resultWalking = await addressService.FindRouteLenght(new FindRouteModel(MapPlace(origins), MapPlace(destinations), TravelMode.Walking));
                foreach (var routeResult in resultDriving.Routes)
                {

                    Db.Model.Measure driving = GetMesasure(resultDriving, routeResult.Origin.OriginalAddress, routeResult.Destination.OriginalAddress, Db.Model.TravelMode.Driving);
                    Db.Model.Measure tranzit = GetMesasure(resultTransit, routeResult.Origin.OriginalAddress, routeResult.Destination.OriginalAddress, Db.Model.TravelMode.Transit);
                    Db.Model.Measure Bicycling = GetMesasure(resultBicyclink, routeResult.Origin.OriginalAddress, routeResult.Destination.OriginalAddress, Db.Model.TravelMode.Bicycling);
                    Db.Model.Measure Walking = GetMesasure(resultWalking, routeResult.Origin.OriginalAddress, routeResult.Destination.OriginalAddress, Db.Model.TravelMode.Walking);

                    jsonDb.Routes.Add(new Db.Model.Route()
                    {
                        Id = currentRouteId++,
                        DestinationId = destinations.Single(item => item.AddressGoogleId == routeResult.Destination.OriginalAddress).Id,
                        OriginId = origins.Single(item => item.AddressGoogleId == routeResult.Origin.OriginalAddress).Id,
                        MeasuresLenght = new List<Db.Model.Measure>()
                        {
                            driving, tranzit, Bicycling, Walking
                        }
                    });
                }
            }
            await jsonDb.SaveToFile($@"C:\data\DbBefere.json");
        }

        [TestMethod]
        public async Task FieldMatrixForAlgorytmsTest()
        {
            MatrixRoute matrixRoute = await GetMatrixRoute($@"C:\data\Test_Db_Nemazat.json", Db.Model.TravelMode.Driving, Db.Model.WeightMode.Duration);

            Assert.AreEqual(matrixRoute.placeIDs[3], 4);
            Assert.AreEqual(matrixRoute.placeIDs.Count(), 5);
            Assert.AreEqual(matrixRoute.matrixRoute[0, 0], 0);
            Assert.AreEqual(matrixRoute.matrixRoute[1, 0], 534);
            Assert.AreEqual(matrixRoute.matrixRoute[2, 3], 934);
            Assert.AreEqual(matrixRoute.matrixRoute[0, 4], 8458);
            Assert.AreEqual(matrixRoute.matrixRoute[4, 0], 8458);
            Assert.AreEqual(matrixRoute.matrixRoute[3, 1], 593);
        }

        [TestMethod]
        public async Task RandomSPAlgoritmTest()
        {
            MatrixRoute matrixRoute = await GetMatrixRoute($@"C:\data\Test_Db_Nemazat.json", Db.Model.TravelMode.Driving, Db.Model.WeightMode.Duration); ;
            var res2 = new TSPAlgorithmRandom().ComputeTSPRoute(matrixRoute);
            var res = new TSPAlgorithmGreedy().ComputeTSPRoute(matrixRoute);

        }

        //Tento test vypíše délky tras uvedách jako příklad v bakalářské práci. jedná se o trasy, které jel ININ
        [TestMethod]
        public async Task GetRouteLenght()
        {
            MatrixRoute matrixRoute1 = await GetMatrixRoute($@"C:\data\FinalDb.json", Db.Model.TravelMode.Transit, Db.Model.WeightMode.Duration);
            var Priklad1 = new Tsp2_OptAlgorithm().GetFullRouteLength(new List<int>() {0, 1 , 9, 14, 3, 21 } ,matrixRoute1.matrixRoute);
            Debug.WriteLine($"Prijkald 1 je o délce {Priklad1/60} minut");

            MatrixRoute matrixRoute1Opt = await GetMatrixRoute($@"C:\data\FinalDb.json", Db.Model.TravelMode.Transit, Db.Model.WeightMode.Duration);
            var Priklad1Opt = new Tsp2_OptAlgorithm().GetFullRouteLength(new List<int>() { 0, 1, 14, 9, 3, 21 }, matrixRoute1Opt.matrixRoute);
            Debug.WriteLine($"Prijkald 1 Optimaluzovaný je o délce {Priklad1Opt/60} minut");

            MatrixRoute matrixRoute2 = await GetMatrixRoute($@"C:\data\FinalDb.json", Db.Model.TravelMode.Transit, Db.Model.WeightMode.Duration);
            var Priklad2 = new Tsp2_OptAlgorithm().GetFullRouteLength(new List<int>() { 0, 1, 9, 3, 11, 21 }, matrixRoute2.matrixRoute);
            Debug.WriteLine($"Prijkald 2 je o délce {Priklad2/60} minut");

            MatrixRoute matrixRoute3 = await GetMatrixRoute($@"C:\data\DbBefere.json", Db.Model.TravelMode.Driving, Db.Model.WeightMode.Duration);
            var Priklad3 = new Tsp2_OptAlgorithm().GetFullRouteLength(new List<int>() { 0, 1, 14, 9, 3, 15, 18, 21 }, matrixRoute3.matrixRoute);
            Debug.WriteLine($"Prijkald 3 je o délce {Priklad3 / 60} minut");

            MatrixRoute matrixRoute4 = await GetMatrixRoute($@"C:\data\DbBefere.json", Db.Model.TravelMode.Transit, Db.Model.WeightMode.Duration);
            var Priklad4 = new Tsp2_OptAlgorithm().GetFullRouteLength(new List<int>() { 0, 1, 2, 14, 9, 3, 18, 21 }, matrixRoute4.matrixRoute);
            Debug.WriteLine($"Prijkald 4 je o délce {Priklad4 / 60} minut");

            MatrixRoute matrixRoute4Opt = await GetMatrixRoute($@"C:\data\DbBefere.json", Db.Model.TravelMode.Transit, Db.Model.WeightMode.Duration);
            var Priklad4Opt = new Tsp2_OptAlgorithm().GetFullRouteLength(new List<int>() { 0, 1, 2, 9, 14, 3, 18, 21 }, matrixRoute4Opt.matrixRoute);
            Debug.WriteLine($"Prijkald 4 optimalizovaný je o délce {Priklad4Opt / 60} minut");

            MatrixRoute matrixRoute5 = await GetMatrixRoute($@"C:\data\DbBefere.json", Db.Model.TravelMode.Transit, Db.Model.WeightMode.Duration);
            var Priklad5 = new Tsp2_OptAlgorithm().GetFullRouteLength(new List<int>() { 0, 20, 14, 3, 15, 21 }, matrixRoute5.matrixRoute);
            Debug.WriteLine($"Prijkald 5 je o délce {Priklad5 / 60} minut");
        }
    }
}
