
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.HttpsPolicy;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Optiroute.Db;
using Optiroute.TSPAlgorithm;
using Optiroute.Web.Services;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Threading.Tasks;

namespace Optiroute.Web
{
    public class Startup
    {
        private readonly IWebHostEnvironment _env;

        public Startup(IConfiguration configuration, IWebHostEnvironment env)
        {
            Configuration = configuration;
            this._env = env;
        }

        public IConfiguration Configuration { get; }

        // This method gets called by the runtime. Use this method to add services to the container.
        public void ConfigureServices(IServiceCollection services)
        {
            services.AddRazorPages().AddRazorRuntimeCompilation();
            services.AddControllersWithViews();
            services.AddSingleton<DbContext>(DbContext.LoadFromFileSync(Path.Combine(_env.WebRootPath, "FinalDb.json")));
            services.AddSingleton<ITSPComputeAlgorithmProviderService>(
                new TSPComputeAlgorithmProviderService(
                    new TSPAlgorithmRandom(),
                    new TSPAlgorithmNearestNeighbour(),
                    new TSPAlgorithmGreedy()
                    ));
            services.AddSingleton<ITSPOptimaliseAlgorithmProviderService>(new TSPOptiAlgorithmProviderService(new Tsp2_OptAlgorithm()));
            services.AddSingleton<IComputeAlgorithmRoutesService, ComputeAlgorithmRoutesService>();
            services.AddSingleton<ICodeBookService, CodeBookService>();
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
        {
            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }
            else
            {
               
                 app.UseExceptionHandler("/Home/Error");
                // The default HSTS value is 30 days. You may want to change this for production scenarios, see https://aka.ms/aspnetcore-hsts.
                 app.UseHsts();
            }

            app.UseHttpsRedirection();
            app.UseStaticFiles();

            app.UseRouting();

            app.UseAuthorization();

            app.UseEndpoints(endpoints =>
            {
                endpoints.MapControllerRoute(
                    name: "default",
                    pattern: "{controller=Home}/{action=Index}/{id?}");
            });
        }
    }
}
